@@ -1,9 +1,12 @@
 use std::collections::HashMap;
+use std::convert::TryInto;
 use std::fmt;
 use std::iter::IntoIterator;
 use std::slice::{Iter, IterMut};
 
+#[cfg(feature = "tty")]
 use crossterm::terminal::size;
+#[cfg(feature = "tty")]
 use crossterm::tty::IsTty;
 use strum::IntoEnumIterator;
 
@@ -12,9 +15,7 @@ use crate::column::Column;
 use crate::row::Row;
 use crate::style::presets::ASCII_FULL;
 use crate::style::{ColumnConstraint, ContentArrangement, TableComponent};
-use crate::utils::arrangement::arrange_content;
-use crate::utils::borders::draw_borders;
-use crate::utils::format::format_content;
+use crate::utils::build_table;
 
 /// This is the main interface for building a table.
 /// Each table consists of [Rows](Row), which in turn contain [Cells](crate::cell::Cell).
@@ -36,11 +37,7 @@ pub struct Table {
 
 impl fmt::Display for Table {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        let display_info = arrange_content(self);
-        let content = format_content(&self, &display_info);
-        let lines = draw_borders(&self, content, &display_info);
-
-        write!(f, "{}", lines.join("\n"))
+        write!(f, "{}", self.lines().collect::<Vec<_>>().join("\n"))
     }
 }
 
@@ -73,15 +70,16 @@ impl Table {
     /// This is an alternative `fmt` function, which simply removes any trailing whitespaces.
     /// Trailing whitespaces often occur, when using tables without a right border.
     pub fn trim_fmt(&self) -> String {
-        let display_info = arrange_content(self);
-        let content = format_content(&self, &display_info);
-        let mut lines = draw_borders(&self, content, &display_info);
-        lines = lines
-            .iter()
+        self.lines()
             .map(|line| line.trim_end().to_string())
-            .collect();
+            .collect::<Vec<_>>()
+            .join("\n")
+    }
 
-        lines.join("\n")
+    /// This is an alternative to `fmt`, but rather returns an iterator to each line, rather than
+    /// one String separated by newlines.
+    pub fn lines(&self) -> impl Iterator<Item = String> {
+        build_table(self)
     }
 
     /// Set the header row of the table. This is usually the title of each column.\
@@ -141,12 +139,25 @@ impl Table {
     ///
     /// If neither is not possible, `None` will be returned.\
     /// This implies that both the [Dynamic](ContentArrangement::Dynamic) mode and the [Percentage](crate::style::ColumnConstraint::Percentage) constraint won't work.
+    #[cfg(feature = "tty")]
     pub fn get_table_width(&self) -> Option<u16> {
         if let Some(width) = self.table_width {
             Some(width)
         } else if self.is_tty() {
-            let (table_width, _) = size().unwrap();
-            Some(table_width)
+            if let Ok((table_width, _)) = size() {
+                Some(table_width)
+            } else {
+                None
+            }
+        } else {
+            None
+        }
+    }
+
+    #[cfg(not(feature = "tty"))]
+    pub fn get_table_width(&self) -> Option<u16> {
+        if let Some(width) = self.table_width {
+            Some(width)
         } else {
             None
         }
@@ -197,6 +208,7 @@ impl Table {
     ///
     /// This function respects the [Table::force_no_tty] function.\
     /// Otherwise we try to determine, if we are on a tty.
+    #[cfg(feature = "tty")]
     pub fn is_tty(&self) -> bool {
         if self.no_tty {
             return false;
@@ -205,6 +217,11 @@ impl Table {
         ::std::io::stdout().is_tty()
     }
 
+    #[cfg(not(feature = "tty"))]
+    pub fn is_tty(&self) -> bool {
+        false
+    }
+
     /// Enforce terminal styling.
     ///
     /// Only useful if you forcefully disabled tty, but still want those fancy terminal styles.
@@ -216,6 +233,7 @@ impl Table {
     /// table.force_no_tty()
     ///     .enforce_styling();
     /// ```
+    #[cfg(feature = "tty")]
     pub fn enforce_styling(&mut self) -> &mut Self {
         self.enforce_styling = true;
 
@@ -232,18 +250,23 @@ impl Table {
     }
 
     /// Convenience method to set a [ColumnConstraint] for all columns at once.
+    /// Constraints are used to influence the way the columns will be arranged.
+    /// Check out their docs for more information.
     ///
-    /// Simply pass any iterable with ColumnConstraints.\
-    /// If more constraints are passed than there are columns, the superfluous constraints will be ignored.
+    /// **Attention:**
+    /// This function should be called after at least one row (or the headers) has been added to the table.
+    /// Before that, the columns won't initialized.
+    ///
+    /// If more constraints are passed than there are columns, any superfluous constraints will be ignored.
     /// ```
-    /// use comfy_table::{Table, ColumnConstraint, ContentArrangement};
+    /// use comfy_table::{Width::*, CellAlignment, ColumnConstraint::*, ContentArrangement, Table};
     ///
     /// let mut table = Table::new();
     /// table.add_row(&vec!["one", "two", "three"])
     ///     .set_content_arrangement(ContentArrangement::Dynamic)
     ///     .set_constraints(vec![
-    ///         ColumnConstraint::MaxWidth(15),
-    ///         ColumnConstraint::MinWidth(20),
+    ///         UpperBoundary(Fixed(15)),
+    ///         LowerBoundary(Fixed(20)),
     /// ]);
     /// ```
     pub fn set_constraints<T: IntoIterator<Item = ColumnConstraint>>(
@@ -433,7 +456,8 @@ impl Table {
     /// Get a mutable iterator over all columns.
     ///
     /// ```
-    /// use comfy_table::{Table, ColumnConstraint};
+    /// use comfy_table::{Width::*, ColumnConstraint::*, Table};
+    ///
     /// let mut table = Table::new();
     /// table.add_row(&vec!["First", "Second", "Third"]);
     ///
@@ -442,9 +466,9 @@ impl Table {
     /// // second -> max width of 8
     /// // third -> fixed width of 10
     /// let constraints = vec![
-    ///     ColumnConstraint::MinWidth(10),
-    ///     ColumnConstraint::MaxWidth(8),
-    ///     ColumnConstraint::Width(10),
+    ///     LowerBoundary(Fixed(10)),
+    ///     UpperBoundary(Fixed(8)),
+    ///     Absolute(Fixed(10)),
     /// ];
     ///
     /// // Add the constraints to their respective column
@@ -549,18 +573,19 @@ impl Table {
     fn adjust_max_column_widths(&mut self, row: &Row) {
         let max_widths = row.max_content_widths();
         for (index, width) in max_widths.iter().enumerate() {
+            let width = (*width).try_into().unwrap_or(u16::MAX);
             // We expect this column to exist, since we autoenerate columns just before calling this function
             let mut column = self.columns.get_mut(index).unwrap();
-            if column.max_content_width < *width as u16 {
-                column.max_content_width = *width as u16;
+            if column.max_content_width < width {
+                column.max_content_width = width;
             }
         }
     }
 }
 
-/// This is an iterator over all cells of a specific column.
-/// A dedicated struct is necessary, since data is usually handled by rows and thereby stored in
-/// `Table::rows`. That's why this iterator also has to be implemented on the Table struct.
+/// An iterator over cells of a specific column.
+/// A dedicated struct is necessary, as data is usually handled by rows and thereby stored in
+/// `Table::rows`. This type is returned by [Table::column_cells_iter].
 pub struct ColumnCellIter<'a> {
     rows: &'a [Row],
     column_index: usize,
